@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,27 +21,31 @@
 
 $this->addJsFile('gtlc.js');
 $this->addJsFile('flickerfreescreen.js');
+$this->addJsFile('layout.mode.js');
 
 (new CWidget())
 	->setTitle(_('Web monitoring'))
-	->setControls((new CForm('get'))
-		->cleanItems()
-		->addVar('action', 'web.view')
-		->addVar('fullscreen', $data['fullscreen'] ? '1' : null)
-		->addItem((new CList())
-			->addItem([
-				new CLabel(_('Group'), 'groupid'),
-				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-				$data['pageFilter']->getGroupsCB()
-			])
-			->addItem([
-				new CLabel(_('Host'), 'hostid'),
-				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-				$data['pageFilter']->getHostsCB()
-			])
-			->addItem(get_icon('fullscreen', ['fullscreen' => $data['fullscreen']]))
-		)
-	)
+	->setWebLayoutMode(CView::getLayoutMode())
+	->setControls((new CList([
+		(new CForm('get'))
+			->cleanItems()
+			->setAttribute('aria-label', _('Main filter'))
+			->addVar('action', 'web.view')
+			->addItem((new CList())
+				->addItem([
+					new CLabel(_('Group'), 'groupid'),
+					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
+					$data['pageFilter']->getGroupsCB()
+				])
+				->addItem([
+					new CLabel(_('Host'), 'hostid'),
+					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
+					$data['pageFilter']->getHostsCB()
+				])
+			),
+		(new CTag('nav', true, get_icon('fullscreen')))
+			->setAttribute('aria-label', _('Content controls'))
+		])))
 	->addItem(
 		CScreenBuilder::getScreen([
 			'resourcetype' => SCREEN_RESOURCE_HTTPTEST,
@@ -52,7 +56,6 @@ $this->addJsFile('flickerfreescreen.js');
 			'page' => $data['page'],
 			'data' => [
 				'hosts_selected' => $data['pageFilter']->hostsSelected,
-				'fullscreen' => $data['fullscreen'],
 				'sort' => $data['sort'],
 				'sortorder' => $data['sortorder'],
 				'groupid' => $data['pageFilter']->groupid
