@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,11 +34,11 @@ function screen_resources($resource = null) {
 		SCREEN_RESOURCE_MAP => _('Map'),
 		SCREEN_RESOURCE_PLAIN_TEXT => _('Plain text'),
 		SCREEN_RESOURCE_SCREEN => _('Screen'),
-		SCREEN_RESOURCE_SERVER_INFO => _('Status of Zabbix'),
+		SCREEN_RESOURCE_SERVER_INFO => _('System information'),
 		SCREEN_RESOURCE_SIMPLE_GRAPH => _('Simple graph'),
 		SCREEN_RESOURCE_HOSTGROUP_TRIGGERS => _('Host group issues'),
 		SCREEN_RESOURCE_HOST_TRIGGERS => _('Host issues'),
-		SCREEN_RESOURCE_SYSTEM_STATUS => _('System status'),
+		SCREEN_RESOURCE_SYSTEM_STATUS => _('Problems by severity'),
 		SCREEN_RESOURCE_TRIGGER_INFO => _('Trigger info'),
 		SCREEN_RESOURCE_TRIGGER_OVERVIEW => _('Trigger overview'),
 		SCREEN_RESOURCE_URL => _('URL'),
@@ -317,6 +317,12 @@ function get_slideshow_by_slideshowid($slideshowid, $permission) {
 function add_slideshow($data) {
 	$user_data = CWebUser::$data;
 
+	if (!validateTimeUnit($data['delay'], 1, SEC_PER_DAY, false, $error)) {
+		error(_s('Incorrect value for field "%1$s": %2$s.', 'delay', $error));
+
+		return false;
+	}
+
 	// Validate slides.
 	if (empty($data['slides'])) {
 		error(_('Slide show must contain slides.'));
@@ -324,6 +330,14 @@ function add_slideshow($data) {
 		return false;
 	}
 
+	foreach ($data['slides'] as $slide) {
+		if (!validateTimeUnit($slide['delay'], 0, SEC_PER_DAY, false, $error)) {
+			error(_s('Incorrect value for field "%1$s": %2$s.', 'delay', $error));
+
+			return false;
+		}
+	}
+
 	// Validate screens.
 	$screenids = zbx_objectValues($data['slides'], 'screenid');
 
@@ -416,11 +430,6 @@ function add_slideshow($data) {
 	foreach ($data['slides'] as $slide) {
 		$slideid = get_dbid('slides', 'slideid');
 
-		// set default delay
-		if (empty($slide['delay'])) {
-			$slide['delay'] = 0;
-		}
-
 		$result = DBexecute(
 			'INSERT INTO slides (slideid,slideshowid,screenid,step,delay)'.
 			' VALUES ('.zbx_dbstr($slideid).','.zbx_dbstr($slideshowid).','.zbx_dbstr($slide['screenid']).','.($i++).','.zbx_dbstr($slide['delay']).')'
@@ -437,6 +446,12 @@ function add_slideshow($data) {
 function update_slideshow($data) {
 	$user_data = CWebUser::$data;
 
+	if (!validateTimeUnit($data['delay'], 1, SEC_PER_DAY, false, $error)) {
+		error(_s('Incorrect value for field "%1$s": %2$s.', 'delay', $error));
+
+		return false;
+	}
+
 	// Validate slides.
 	if (empty($data['slides'])) {
 		error(_('Slide show must contain slides.'));
@@ -444,6 +459,14 @@ function update_slideshow($data) {
 		return false;
 	}
 
+	foreach ($data['slides'] as $slide) {
+		if (!validateTimeUnit($slide['delay'], 0, SEC_PER_DAY, false, $error)) {
+			error(_s('Incorrect value for field "%1$s": %2$s.', 'delay', $error));
+
+			return false;
+		}
+	}
+
 	// validate screens.
 	$screenids = zbx_objectValues($data['slides'], 'screenid');
 
@@ -625,7 +648,6 @@ function update_slideshow($data) {
 	$slidesToDel = zbx_toHash($slidesToDel);
 	$step = 0;
 	foreach ($data['slides'] as $slide) {
-		$slide['delay'] = $slide['delay'] ? $slide['delay'] : 0;
 		if (isset($db_slides[$slide['slideid']])) {
 			// update slide
 			if ($db_slides[$slide['slideid']]['delay'] != $slide['delay'] || $db_slides[$slide['slideid']]['step'] != $step) {
